#include "RTClib.h"
#include <FastLED.h>

#define NUM_LEDS 96
#define DATA_PIN 3
#define LDR A0

RTC_DS1307 rtc;

boolean timeChange = false;
boolean printTime = false; // Set to true if you want to see output in the console. Helpful for debugging.
boolean redDown = true;
boolean greenDown = false;
boolean blueDown = false;
boolean cycle = false; // Set true if you want clock colors to cycle
boolean highlight = true; // Set true to highlight 'last digit'.

// Locations of the start and end of each group of time
const int SECOND_1_LOCATION = 0;
const int HOUR_2_START_LOCATION = 1;
const int HOUR_2_END_LOCATION = 8;
const int HOUR_1_START_LOCATION = 9;
const int HOUR_1_END_LOCATION = 20;
const int MINUTE_2_START_LOCATION = 21;
const int MINUTE_2_END_LOCATION = 42;
const int MINUTE_1_START_LOCATION = 43;
const int MINUTE_1_END_LOCATION = 66;
const int SECOND_2_START_LOCATION = 67;
const int SECOND_2_END_LOCATION = 95;
const int LEDS_PER_HOUR_1 = 3;
const int LEDS_PER_HOUR_2 = 2;
const int LEDS_PER_MINUTE_1 = 6;
const int LEDS_PER_MINUTE_2 = 2;

// Multipliers used to split up time
const int MULTIPLIER_FIVE = 5;
const int MULTIPLIER_TWO = 2;

const int START_UP_DELAY = 1; // Change this to speed up or slow down startup animation
const int CYCLE_SPEED = 1; // Change the rate here for color changing cycle (must be above 1)

// Declair variables
int lastSecond = 0;
int currentHour = 0;
int currentMinute = 0;
int currentSecond = 0;
int hour1 = 0;
int hour2 = 0;
int minute1 = 0;
int minute2 = 0;
int second1 = 0;
int second2 = 0;
int cycleCount = 1;
float fadeValue = 255;
float fadeCheck = 255;
uint8_t bright = 255;

int numberOfAnimations = 5;
int randomness = 0;

// Set Colors
uint8_t red = 0;
uint8_t green = 0;
uint8_t blue = 255;
uint8_t highlight_red = 60;
uint8_t highlight_green = 60;
uint8_t highlight_blue = 255;

// Define the array of leds
CRGB leds[NUM_LEDS];

void setup() {

  Serial.begin(19200);
  FastLED.addLeds<WS2811, DATA_PIN, RGB>(leds, NUM_LEDS);
  LEDS.setBrightness(bright);
  FastLED.clear();
  rtc.begin();


  // Uncomment line below to set time.
//     rtc.adjust(DateTime(2020, 2, 19, 23, 59, 50));
//     rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  // Startup animation
  animate(randomness);
}

void loop() {

  // Get time
  DateTime now = rtc.now();
  currentHour = now.hour();
  currentMinute = now.minute();
  currentSecond = now.second();
  timeChange = false;

  // Use these to manually set time without RTC. Helpful for debugging
  //    currentHour = 5;
  //    currentMinute = 30;
  //    currentSecond = 30;

  //   Reset all bits to zero
  for (int i = SECOND_1_LOCATION; i <= SECOND_2_END_LOCATION; i++)
  {
    leds[i] = CRGB::Black;
  }

  // Set Hour
  // Set hour 1
  hour1 = (currentHour % MULTIPLIER_FIVE) * LEDS_PER_HOUR_1; // This will count the total LEDs of the time unit to light up

  for (int i = HOUR_1_START_LOCATION; i < (HOUR_1_START_LOCATION + hour1); i++)
  {
    leds[i] = CRGB( red, green, blue);
  }

  if (highlight == true && hour1 > 0)// && hour1 < 12)
  {
    for (int i = (HOUR_1_START_LOCATION + hour1 - 1); i >= (HOUR_1_START_LOCATION + hour1 - LEDS_PER_HOUR_1); i--)
    {
      leds[i] = CRGB( highlight_red, highlight_green, highlight_blue);
    }
  }

  // Set hour 2
  hour2 = (currentHour / MULTIPLIER_FIVE) * LEDS_PER_HOUR_2; // This will count the total LEDs of the time unit to light up

  for (int i = HOUR_2_START_LOCATION; i < (HOUR_2_START_LOCATION + hour2); i++)
  {
    leds[i] = CRGB( red, green, blue);
  }

  if (highlight == true && hour2 > 0)// && hour2 < 8)
  {
    for (int i = (HOUR_2_START_LOCATION + hour2 - 1); i >= (HOUR_2_START_LOCATION + hour2 - LEDS_PER_HOUR_2); i--)
    {
      leds[i] = CRGB( highlight_red, highlight_green, highlight_blue);
    }
  }

  // Set Minute
  // Set minute 1
  minute1 = (currentMinute % MULTIPLIER_FIVE) * LEDS_PER_MINUTE_1; // This will count the total LEDs of the time unit to light up

  for (int i = MINUTE_1_START_LOCATION; i < (MINUTE_1_START_LOCATION + minute1); i++)
  {
    leds[i] = CRGB( red, green, blue);
  }

  if (highlight == true && minute1 > 0)// && minute1 < 24)
  {
    for (int i = (MINUTE_1_START_LOCATION + minute1 - 1); i >= (MINUTE_1_START_LOCATION + minute1 - LEDS_PER_MINUTE_1); i--)
    {
      leds[i] = CRGB( highlight_red, highlight_green, highlight_blue);
    }
  }

  // Set minute 2
  minute2 = (currentMinute / MULTIPLIER_FIVE) * LEDS_PER_MINUTE_2; // This will count the total LEDs of the time unit to light up

  for (int i = MINUTE_2_START_LOCATION; i < (MINUTE_2_START_LOCATION + minute2); i++)
  {
    leds[i] = CRGB( red, green, blue);

  }

  if (highlight == true && minute2 > 0)// && minute2 < 22)
  {
    for (int i = (MINUTE_2_START_LOCATION + minute2 - 1); i >= (MINUTE_2_START_LOCATION + minute2 - LEDS_PER_MINUTE_2); i--)
    {
      leds[i] = CRGB( highlight_red, highlight_green, highlight_blue);
    }
  }

  // Set Second
  if (currentSecond != lastSecond)
  {
    timeChange = true;
  }

  // Set second 1
  second1 = currentSecond % MULTIPLIER_TWO;

  if (second1 == 1)
  {
    leds[SECOND_1_LOCATION] = CRGB( red, green, blue);
  }

  // Set second 2
  second2 = currentSecond / MULTIPLIER_TWO;

  for (int i = SECOND_2_START_LOCATION; i < (SECOND_2_START_LOCATION + second2); i++)
  {
    leds[i] = CRGB( red, green, blue);
  }

  if (highlight == true && second2 > 0)// && second2 < 29)
  {
    for (int i = (SECOND_2_START_LOCATION + second2 - 1); i >= (SECOND_2_START_LOCATION + second2 - 1); i--)
    {
      leds[i] = CRGB( highlight_red, highlight_green, highlight_blue);
    }
  }
  lastSecond = currentSecond;

  // Count cycles of the program and call the setColor function to change the color of the LEDs ever CYCLE_SPEED cycles.
  if (cycleCount < CYCLE_SPEED)
  {
    cycleCount++;
  }

  else if (cycleCount == CYCLE_SPEED && cycle == true)
  {
    cycleCount++;
    setColor(cycle);
  }

  else
  {
    cycleCount = 0;
  }

  // Animate every hour
  randomness = random(numberOfAnimations);
  if (currentMinute == 0 && currentSecond == 0)
  {
    animate(randomness);
  }

  // This equation is creating using interpolation to determine the brightness.
  // I suggest leaving it commented unless you find the LEDs too bright at night.
  // If you want to change the brightness of your lights and want to create your own equation,
  // search 'Interpolation' online and jump feet first into the world of numerial analysis.
  
  //  fadeValue = analogRead(LDR) * -.18 + 333.71;
  //  float readin = analogRead(LDR);
  //  Serial.println(readin);
  //  Serial.println(fadeValue);
  //  if (fadeValue > bright)
  //  {
  //    fadeValue = bright;
  //  }
  //  else if (fadeValue < 150)
  //  {
  //    fadeValue = 150;
  //  }
  //  LEDS.setBrightness(fadeValue);

  FastLED.show();

  // Print current time to the console
  if (timeChange == true && printTime == true)
  {
    printToConsole();
  }
}

// Animation function add more animations here as you wish
void animate(int select)
{
  if (select == 0)
  {
    for (int i = SECOND_1_LOCATION; i <= SECOND_2_END_LOCATION; i++)
    {
      leds[i] = CRGB( red, green, blue);
      FastLED.show();

      delay(START_UP_DELAY);
    }

    for (int i = SECOND_2_END_LOCATION; i >= SECOND_1_LOCATION; i--)
    {
      leds[i] = CRGB::Black;
      FastLED.show();

      delay(START_UP_DELAY);
    }
  }

  else if (select == 1)
  {
    for (int i = 0; i < 250; i++)
    {
      int light = random(95);

      leds[light] = CRGB( red, green, blue);

      FastLED.show();
    }
  }

  else if (select == 2)
  {
    leds[0] = CRGB( red, green, blue);

    for (int i = 0; i <= SECOND_2_END_LOCATION - SECOND_2_START_LOCATION; i++)
    {
      leds[SECOND_2_START_LOCATION + i] = CRGB( red, green, blue);

      if (i <= (MINUTE_1_END_LOCATION - MINUTE_1_START_LOCATION))
      {
        leds[MINUTE_1_START_LOCATION + i] = CRGB( red, green, blue);
      }

      if (i <= (MINUTE_2_END_LOCATION - MINUTE_2_START_LOCATION))
      {
        leds[MINUTE_2_START_LOCATION + i] = CRGB( red, green, blue);
      }

      if (i <= (HOUR_1_END_LOCATION - HOUR_1_START_LOCATION))
      {
        leds[HOUR_1_START_LOCATION + i] = CRGB( red, green, blue);
      }

      if (i <= (HOUR_2_END_LOCATION - HOUR_2_START_LOCATION))
      {
        leds[HOUR_2_START_LOCATION + i] = CRGB( red, green, blue);
      }

      delay(34);
      FastLED.show();
    }
  }

  else if (select == 3)
  {
    leds[0] = CRGB( red, green, blue);

    for (int i = 0; i <= SECOND_2_END_LOCATION - SECOND_2_START_LOCATION; i++)
    {
      leds[SECOND_2_END_LOCATION - i] = CRGB( red, green, blue);

      if (i <= (MINUTE_1_END_LOCATION - MINUTE_1_START_LOCATION))
      {
        leds[MINUTE_1_END_LOCATION - i] = CRGB( red, green, blue);
      }

      if (i <= (MINUTE_2_END_LOCATION - MINUTE_2_START_LOCATION))
      {
        leds[MINUTE_2_END_LOCATION - i] = CRGB( red, green, blue);
      }

      if (i <= (HOUR_1_END_LOCATION - HOUR_1_START_LOCATION))
      {
        leds[HOUR_1_END_LOCATION - i] = CRGB( red, green, blue);
      }

      if (i <= (HOUR_2_END_LOCATION - HOUR_2_START_LOCATION))
      {
        leds[HOUR_2_END_LOCATION - i] = CRGB( red, green, blue);
      }

      delay(34);
      FastLED.show();
    }
  }

  else if (select == 4)
  {
    for (int i = SECOND_2_END_LOCATION; i >= SECOND_1_LOCATION; i--)
    {
      leds[i] = CRGB::Black;
    }
    FastLED.show();
    delay(200);

    for (int i = SECOND_1_LOCATION; i <= SECOND_2_END_LOCATION; i++)
    {
      leds[i] = CRGB( red, green, blue);
    }
    FastLED.show();
    delay(200);

    for (int i = SECOND_2_END_LOCATION; i >= SECOND_1_LOCATION; i--)
    {
      leds[i] = CRGB::Black;
    }
    FastLED.show();
    delay(200);

    for (int i = SECOND_1_LOCATION; i <= SECOND_2_END_LOCATION; i++)
    {
      leds[i] = CRGB( red, green, blue);
    }
    FastLED.show();
    delay(200);

    for (int i = SECOND_2_END_LOCATION; i >= SECOND_1_LOCATION; i--)
    {
      leds[i] = CRGB::Black;
    }
    FastLED.show();
    delay(200);
  }
}

// Color cycling function
void setColor(boolean cycleColors)
{
  if (cycleColors == true)
  {
    if (redDown == true && greenDown == false)
    {
      red++;
      green--;
      if (green <= 0)
      {
        red = 255;
        redDown = false;
        greenDown = true;
      }
    }
    else if (greenDown == true && blueDown == false)
    {
      green++;
      blue--;
      if (blue <= 0)
      {
        green = 255;
        greenDown = false;
        blueDown = true;
      }
    }
    else if (blueDown == true && redDown == false)
    {
      blue++;
      red--;
      if (red <= 0)
      {
        blue = 255;
        blueDown = false;
        redDown = true;
      }
    }
  }

  else
  {
    red = 0;
    green = 0;
    blue = 255;
  }

}

// Print to Serial Monitor funciton
void printToConsole()
{
  Serial.print("Current Time: ");
  Serial.print(currentHour);
  Serial.print(":");
  Serial.print(currentMinute);
  Serial.print(":");
  Serial.println(currentSecond);
  Serial.println(" ");


  for (int i = HOUR_2_START_LOCATION; i <= HOUR_2_END_LOCATION; i++)
  {
    Serial.print(leds[i]);
    if (i % 2 == 0)
    {
      Serial.print("  ");
    }
  }
  Serial.println(" ");

  for (int i = HOUR_1_START_LOCATION; i <= HOUR_1_END_LOCATION; i++)
  {
    Serial.print(leds[i]);
    if (((i - HOUR_1_START_LOCATION + 1) % 3) == 0)
    {
      Serial.print(" ");
    }
  }
  Serial.println("  ");

  for (int i = MINUTE_2_START_LOCATION; i <= MINUTE_2_END_LOCATION; i++)
  {
    Serial.print(leds[i]);
    if (((i - MINUTE_2_START_LOCATION) + 1) % 2 == 0)
    {
      Serial.print(" ");
    }
  }
  Serial.println(" ");


  for (int i = MINUTE_1_START_LOCATION; i <= MINUTE_1_END_LOCATION; i++)
  {
    Serial.print(leds[i]);
    if (((i - MINUTE_1_START_LOCATION) + 1) % 6 == 0)
    {
      Serial.print("  ");
    }
  }

  Serial.println(" ");

  for (int i = SECOND_2_START_LOCATION; i <= SECOND_2_END_LOCATION; i++)
  {
    Serial.print(leds[i]);
    Serial.print(" ");
  }

  Serial.println(" ");

  Serial.println(leds[SECOND_1_LOCATION]);

  Serial.println();

  for (int i = 0; i < NUM_LEDS; i++)
  {
    Serial.print(leds[i]);
  }

  Serial.println();
  Serial.println();
}
